
//#include "metalShaderTypes.h"

typedef struct
{
	float4 burnColor;
} fxVars;

constant float speed=1.;
constant float angle=2.;
constant float power=2.;


vertex vertexOut vertexFunc(uint vertexID [[ vertex_id ]],const device txShaderVers* in[[ buffer(0) ]],constant txShaderUniforms& uniforms[[ buffer(3) ]],constant fxVars& vars[[ buffer(2)]])
{
    vertexOut out;

    out.position=in[vertexID].pos*uniforms.worldViewProj;

    out.tex1=in[vertexID].tex1;
	out.tex2=in[vertexID].tex2;
	out.texMask=in[vertexID].texMask;
    return out;
}

fragment float4 fragmentFunc(vertexOut in [[stage_in]]
							 ,texture2d<half> inputTex0 [[ texture(0) ]]
							 ,texture2d<half> inputTex1 [[ texture(1) ]]
							 ,texture2d<half> inputTexMask [[ texture(2) ]]
							 ,constant txShaderUniforms& u[[ buffer(2) ]]
							 ,constant fxVars& vars[[ buffer(1)]])
{
	float fsMix;
	if(u.transitionProgress<=.5)
		fsMix=u.transitionProgress*2.;
	else
		fsMix=(1.-u.transitionProgress)*2.;
	
	vec2 p = in.tex1;
	
	float t = pow(u.transitionProgressLinear, power)*speed;
	p = p -0.5;
	for (int i = 0; i < 7; i++)
	{
		p = vec2(sin(t)*p.x + cos(t)*p.y, sin(t)*p.y - cos(t)*p.x);
		t += angle;
		p = abs(mod(p, 2.0) - 1.0);
	}
	abs(mod(p, 1.0));


	vec4 colorA=tex(inputTex0,in.tex1);
	vec4 colorB=colorA*u.transitionAlpha*u.baseAlpha;

	vec4 colorA2=tex(inputTex0,p);

	colorA=mix(colorA,colorA2,1.0 - 2.0*abs(u.transitionProgressLinear - 0.5));


	float4 outColor=mix(colorB,colorA,u.mix);

#ifdef USE_MASK
	outColor.a*=float(inputTexMask.sample(linearSampler,in.texMask).a);
#endif

	return outColor;
}
